#pragma once

namespace flight_plan {



//========================================================================================
// FlightPlan
//========================================================================================

class FlightPlan{
public:
    FlightPlan();

    void setScenario(Scenario &scenario)          {_scenario = scenario;}
    void setProgress(vector<int> &progress)       {_progress = progress;}

    const Scenario          &getScenario(void)          const {return _scenario;}
    const BoostLineString   &getWaypointsENU(void)      const {return _waypointsENU;}
    const GeoPoint2DList    &getWaypoints(void)         const {return _waypoints;}
    const vector<uint64_t>  &getArrivalPathIdx(void)    const {return _arrivalPathIdx;}
    const vector<uint64_t>  &getReturnPathIdx(void)     const {return _returnPathIdx;}
#ifndef NDEBUG
    const BoostLineString   &getPathVertices(void)      const {return _PathVertices;}
#endif

    bool                            generate(double lineDistance, double minTransectLength);
    const vector<BoostLineString>  &getTransects() const {return _transects;}

    string errorString;

private:
    // Search Filter to speed up routing.SolveWithParameters(...);
    // Found here: http://www.lia.disi.unibo.it/Staff/MicheleLombardi/or-tools-doc/user_manual/manual/ls/ls_filtering.html
    class SearchFilter;

    struct RoutingDataModel;

    bool _generateTransects(double lineDistance, double minTransectLength);
    void _generateRoutingModel(const BoostLineString &vertices,
                               const BoostPolygon &polygonOffset,
                               size_t n0,
                               RoutingDataModel &dataModel,
                               Matrix<double> &graph);

    Scenario                                _scenario;
    BoostLineString                         _waypointsENU;
    GeoPoint2DList                          _waypoints;
    vector<BoostLineString>                 _transects;
    vector<int>                             _progress;
    vector<uint64_t>                        _arrivalPathIdx;
    vector<uint64_t>                        _returnPathIdx;

#ifndef NDEBUG
    BoostLineString                         _PathVertices;
#endif
};

}
